#include <stdio.h>  
#include <cuda.h>  
#include <time.h>

#include <cstdio>

#include "common.h"

#define BLOCKDIM 16

__global__ void matkernel(Matrix a, Matrix b, Matrix c)
{
   int i = threadIdx.x;
   int j = threadIdx.y;
   
   c.data[(c.cols * i) + j] = a.data[(a.cols * i) + j] + 
                              b.data[(b.cols * i) + j];
}

void checkCUDAError(const char *msg)
{
    cudaError_t err = cudaGetLastError();
    if( cudaSuccess != err) 
    {
        fprintf(stderr, "Cuda error: %s: %s.\n", msg, 
                                  cudaGetErrorString( err) );
        exit(EXIT_FAILURE);
    }                         
}

void cudamat (Matrix & a, Matrix & b, Matrix & c)
{
   Matrix d_a;
   d_a.rows = a.rows; 
   d_a.cols = a.cols;
   size_t size = sizeof (float) * d_a.rows * d_a.cols;
   cudaMalloc((void **) &d_a.data, size);
   checkCUDAError ("matkernel");

   cudaMemcpy(d_a.data, a.data, size, cudaMemcpyHostToDevice);

   checkCUDAError ("matkernel");

   Matrix d_b;
   d_b.rows = b.rows; 
   d_b.cols = b.cols;
   size = sizeof (float) * d_b.rows * d_b.cols;
   cudaMalloc((void **) &d_b.data, size);

   checkCUDAError ("matkernel");

   cudaMemcpy(d_b.data, b.data, size, cudaMemcpyHostToDevice);
   
   checkCUDAError ("matkernel");

   Matrix d_c;
   d_c.rows = c.rows;
   d_c.cols = c.cols;
   size = sizeof (float) * d_c.rows * d_c.cols;
   cudaMalloc((void **) &d_c.data, size);

   // kernel
   int dev = 0;
   cudaDeviceProp deviceProp;
   cudaGetDeviceProperties(&deviceProp, dev);

   if (deviceProp.maxThreadsPerBlock < (c.rows * c.cols))
   {
     fprintf(stderr, "too many ThreadsPerBlock \n");
     exit(1);
   }

   if ((deviceProp.maxThreadsDim[0] < c.rows) or 
       (deviceProp.maxThreadsDim[1] < c.cols))
   {
     fprintf(stderr, "too many ThreadsPerBlock \n");
     exit(1);
   }

   dim3 dimblock (c.rows, c.cols); 
   matkernel <<< 1, dimblock >>> (d_a, d_b, d_c);

   checkCUDAError ("matkernel");

   //cutilCheckMsg("");

   cudaThreadSynchronize();

   size = sizeof(float) * d_c.rows * d_c.cols;

   cudaMemcpy(c.data, d_c.data, size, 
		cudaMemcpyDeviceToHost);

   checkCUDAError ("matkernel");

   cudaThreadSynchronize();

   cudaFree (d_c.data);
   cudaFree (d_a.data);
   cudaFree (d_b.data);
}
