#include <iostream>
#include <cmath>
#include <time.h>
#include <cstdlib>

#include "common.h"

#define DIM1 20
#define DIM2 22

int main (int argc, char ** argv)
{

  Matrix a, b, c;

  if (argc != 3)
  {
    a.rows = DIM1;
    a.cols = DIM2;
  }
  else
  {
    a.rows = atoi(argv[1]);
    a.cols = atoi(argv[2]);
  }

  std::cout << "Block DIM: " << a.rows * a.cols << std::endl;

  b.cols = a.cols;
  b.rows = a.rows;
  c.rows = a.rows;
  c.cols = a.cols;

  a.data = new float [a.rows * a.cols];
  b.data = new float [b.rows * b.cols];
  c.data = new float [c.rows * c.cols];

  for (int i=0; i<a.rows; i++)
    for (int j=0; j<a.cols; j++)
      a.data[(a.cols * i) + j] = (i*j);

  for (int i=0; i<b.rows; i++)
    for (int j=0; j<b.cols; j++)
      b.data[(b.cols * i) + j] = 0.0;

  double t, ct;

  ct = clock ();
  t = tempo ();
  cudamat (a, b, c);
  t = tempo () - t;
  ct = clock () - ct;

  std::cout << "Time for cudamatxmat: " << t << std::endl;
  std::cout << "CPU Time for cudamatxmat: " << (ct/CLOCKS_PER_SEC) << 
    std::endl;

  Matrix c_t;
  c_t.rows = c.rows;
  c_t.cols = c.cols;
  c_t.data = new float [c_t.rows * c_t.cols];

 for (int i=0; i<a.rows; i++)
    for (int j=0; j<a.cols; j++)
      a.data[(a.cols * i) + j] = (i*j);

  for (int i=0; i<b.rows; i++)
    for (int j=0; j<b.cols; j++)
      b.data[(b.cols * i) + j] = 0.0;
  
  ct = clock ();
  t = tempo ();
  mat (a, b, c_t);
  t = tempo () - t;
  ct = clock () - ct;

  std::cout << "Time for matxmat: " << t << std::endl;
  std::cout << "CPU Time for matxmat: " << (ct/CLOCKS_PER_SEC) << 
    std::endl;

  for (int i=0; i<c.rows; i++)
    for (int j=0; j<c.cols; j++)
      if (c.data[(c.cols * i) + j] != c_t.data[(c_t.cols * i) + j])
        std::cerr << c.data[(c.cols * i) + j] << " and " << 
          c_t.data[(c_t.cols * i) + j] << " ==> " << i 
          << ", " << j  << std::endl;

  delete c_t.data;

  delete a.data;
  delete b.data;
  delete c.data;

  return 0;
}
