#include <stdio.h>
#include <assert.h>
#include <cuda.h>

void incrementArrayOnHost(float *a, int N, float val)
{
  int i;
  for (i=0; i < N; i++) 
    a[i] = a[i] + val;
}

__global__ void incrementArrayOnDevice(float *a, int N, float val)
{
  int idx = blockIdx.x*blockDim.x + threadIdx.x;

  if (idx < N) 
    a[idx] = a[idx] + val;
}

int main(void)
{
  float *a_h, *b_h;     
  float *a_d;          
  int i, N = 16;
  size_t size = N*sizeof(float);

  float val = 10.0;

  a_h = (float *)malloc(size);
  b_h = (float *)malloc(size);
  cudaMalloc((void **) &a_d, size);

  for (i=0; i<N; i++) 
    a_h[i] = (float)i;
  
  cudaMemcpy(a_d, a_h, sizeof(float)*N, cudaMemcpyHostToDevice);
  
  incrementArrayOnHost(a_h, N, val);

  int blockSize = 4;
  int nBlocks = N/blockSize + (N%blockSize == 0?0:1);

  incrementArrayOnDevice <<< nBlocks, blockSize >>> (a_d, N, val);
  
  cudaMemcpy(b_h, a_d, sizeof(float)*N, cudaMemcpyDeviceToHost);
  
  for (i=0; i<N; i++) 
    assert(a_h[i] == b_h[i]);
  
  free(a_h); 
  free(b_h); 
  
  cudaFree(a_d); 
}
