#include <stdio.h>
//#include <cutil.h>
#include <cassert>

__global__ void kernel( int *a, int *b, int *c, int dim ) 
{
    int ix  = blockIdx.x*blockDim.x + threadIdx.x;
    int iy  = blockIdx.y*blockDim.y + threadIdx.y;
    int idx = ix*dim + iy;

    c[idx] = 0.0;

    for (int i=0; i<dim; i++)
      c[idx] += a[(ix * dim) + i] * b[(i * dim) + iy];
}

int main(int argc, char *argv[]) 
{
    int dim = 16;
    int num_bytes = dim*dim*sizeof(int);

    int *d_a=0, *h_a=0;
    int *d_b=0, *h_b=0;
    int *d_c=0, *h_c=0;

    h_a = (int*) malloc(num_bytes);
    cudaMalloc( (void**)&d_a, num_bytes );
    if ( (h_a == NULL) || (d_a == NULL) ) 
    {
      fprintf(stderr,"couldn't allocate memory\n");
      return 1;
    }

    for(int row=0; row<dim; row++)
      for(int col=0; col<dim; col++)
        h_a[row*dim+col] = 1 + (int) (10.0 * (rand() / (RAND_MAX + 1.0)));

    cudaMemcpy( d_a, h_a, num_bytes, cudaMemcpyHostToDevice );

    h_b = (int*) malloc (num_bytes);
    cudaMalloc( (void**)&d_b, num_bytes );
    if ( (h_b == NULL) || (d_b == NULL) ) 
    {
      fprintf(stderr,"couldn't allocate memory\n");
      return 1;
    }

    for(int row=0; row<dim; row++)
    {
      for(int col=0; col<dim; col++)
      {
        if (row == col)
          h_b[row*dim+col] = 1;
        else
          h_b[row*dim+col] = 0;
      }
    }
    cudaMemcpy( d_b, h_b, num_bytes, cudaMemcpyHostToDevice );

    h_c = (int*)malloc(num_bytes);
    cudaMalloc( (void**)&d_c, num_bytes );
    if ( (h_c == NULL) || (d_c == NULL) ) 
    {
      fprintf(stderr,"couldn't allocate memory\n");
      return 1;
    }

    dim3 grid, block;
    block.x = 4;
    block.y = 4;
    grid.x = dim / block.x;
    grid.y = dim / block.y;

    kernel<<<grid, block>>>( d_a, d_b, d_c, dim );

    cudaMemcpy( h_c, d_c, num_bytes, cudaMemcpyDeviceToHost );

    for(int row=0; row<dim; row++)
    {
      for(int col=0; col<dim; col++)
      {
        assert (h_a[row*dim+col] == h_c[row*dim+col]);
      }
    }

    free( h_a );
    cudaFree( d_a );

    free( h_b );
    cudaFree( d_b );

    free( h_c );
    cudaFree( d_c );

    return 0;
}

