#include <stdio.h>
#include <assert.h>

void checkCUDAError(const char* msg);

__global__ void reverseArrayBlock(int *d_out, int *d_in, int dim)
{
  int idx = blockIdx.x*blockDim.x + threadIdx.x;
  if (idx < dim)
    d_out[idx] = d_in[dim - 1 - idx];
}

int main( int argc, char** argv) 
{
    int *h_a;
    int dimA = 32*256 * 1024; 

    int *d_b, *d_a;

    int numThreadsPerBlock = 256;

    int numBlocks = dimA / numThreadsPerBlock;  

    size_t memSize = numBlocks * numThreadsPerBlock * sizeof(int);
    h_a = (int *) malloc(memSize);
    cudaMalloc( (void **) &d_a, memSize );
    cudaMalloc( (void **) &d_b, memSize );

    for (int i = 0; i < dimA; ++i)
    {
      h_a[i] = i;
    }

    cudaMemcpy( d_a, h_a, memSize, cudaMemcpyHostToDevice );
    cudaMemset( d_b, 0, memSize );

    dim3 dimGrid(numBlocks);
    dim3 dimBlock(numThreadsPerBlock);
    reverseArrayBlock<<< dimGrid, dimBlock >>>( d_b, d_a, dimA);

    cudaThreadSynchronize();

    checkCUDAError("kernel invocation");

    cudaMemcpy( h_a, d_b, memSize, cudaMemcpyDeviceToHost );

    checkCUDAError("memcpy");

    for (int i = 0; i < dimA; i++)
      assert(h_a[i] == dimA - 1 - i );

    cudaFree(d_a);
    cudaFree(d_b);

    free(h_a);

    printf("Correct!\n");

    return 0;
}

void checkCUDAError(const char *msg)
{
    cudaError_t err = cudaGetLastError();
    if( cudaSuccess != err) 
    {
      fprintf(stderr, "Cuda error: %s: %s.\n", msg, 
                                cudaGetErrorString( err) );
      exit(EXIT_FAILURE);
    }                         
}
